@@ -3,13 +3,22 @@ package SQLite::KeyValueStore::Simple;
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2021-06-18'; # DATE
 our $DIST = 'SQLite-KeyValueStore-Simple'; # DIST
-our $VERSION = '0.001'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
 use 5.010001;
 use strict;
 use warnings;
 use Log::ger;
 
+use Exporter qw(import);
+our @EXPORT_OK = qw(
+                       dump_sqlite_kvstore
+                       list_sqlite_kvstore_keys
+                       get_sqlite_kvstore_value
+                       set_sqlite_kvstore_value
+                       check_sqlite_kvstore_key_exists
+               );
+
 our $db_schema_spec = {
     latest_v => 1,
     install => [
@@ -50,7 +59,7 @@ sub _decode_value {
     if ($encoding eq 'j') {
         require JSON::MaybeXS;
         eval { $decoded = JSON::MaybeXS::decode_json($value) };
-        return [500, "Can't decode JSON value: $@"];
+        return [500, "Can't decode JSON value: $@"] if $@;
     } elsif ($encoding eq 'r') {
         $decoded = $value;
     } elsif ($encoding eq 'h') {
@@ -75,7 +84,7 @@ sub _encode_value {
     } elsif ($encoding eq 'j') {
         require JSON::MaybeXS;
         eval { $encoded = JSON::MaybeXS::encode_json($value) };
-        return [500, "Can't encode JSON value: $@"];
+        return [500, "Can't encode JSON value: $@"] if $@;
     } else {
         return [412, "Can't encode undef/structure to '$encoding', please choose 'j'"]
             if ref $value or !defined($value);
@@ -182,6 +191,35 @@ _
     },
 );
 
+$SPEC{dump_sqlite_kvstore} = {
+    v => 1.1,
+    summary => 'Dump content of key-value store as hash',
+    description => <<'_',
+_
+    args => {
+        %argspecs_common,
+    },
+};
+sub dump_sqlite_kvstore {
+    my %args = @_;
+
+    my ($res, $dbh) = _init(\%args);
+    return $res unless $res->[0] == 200;
+
+    my %hash;
+    my $sth = $dbh->prepare("SELECT key,value,encoding FROM kvstore");
+    $sth->execute;
+    while (my $row = $sth->fetchrow_arrayref) {
+        my $res = _decode_value($row->[1], $row->[2]);
+        if ($res->[0] != 200) {
+            warn "Key '$row->[0]' cannot be decoded: $res->[0] - $res->[1], skipped";
+            next;
+        }
+        $hash{ $row->[0] } = $res->[2];
+    }
+    [200, "OK", \%hash];
+}
+
 $SPEC{list_sqlite_kvstore_keys} = {
     v => 1.1,
     summary => 'List existing keys in the key-value store',
@@ -249,6 +287,7 @@ _
         %argspec1_value,
         %argspecopt_input_encoding,
         %argspecopt_output_encoding,
+        %argspecopt_quiet,
     },
 };
 sub set_sqlite_kvstore_value {
@@ -273,14 +312,19 @@ sub set_sqlite_kvstore_value {
             my $dres = _decode_value($args{value}, $args{input_encoding} // 'r');
             do { $res = $dres; last WORK } unless $dres->[0] == 200;
             my $newval = $dres->[2];
-            my $eres = _encode_value($newval, ref $newval || !defined($newval) ? 'j' : 'r');
+            my $store_encoding = ref $newval || !defined($newval) ? 'j' : 'r';
+            my $eres = _encode_value($newval, $store_encoding);
             my $encoded = $eres->[2];
-            $dbh->do("INSERT OR IGNORE INTO kvstore (key,value,encoding) VALUES (?,'','r')", {}, $args{key});
-            $dbh->do("UPDATE kvstore SET value=? WHERE key=?", {}, $encoded, $args{key});
+            $dbh->do("INSERT OR IGNORE INTO kvstore (key,value,encoding) VALUES (?,'','')", {}, $args{key});
+            $dbh->do("UPDATE kvstore SET value=?,encoding=? WHERE key=?", {}, $encoded, $store_encoding, $args{key});
         }
     }
     $dbh->commit;
-    _encode_value($oldval, $args{output_encoding});
+    if ($args{quiet}) {
+        [200, "OK"];
+    } else {
+        _encode_value($oldval, $args{output_encoding});
+    }
 }
 
 $SPEC{check_sqlite_kvstore_key_exists} = {
@@ -320,13 +364,14 @@ SQLite::KeyValueStore::Simple - A simple key-value store using SQLite
 
 =head1 VERSION
 
-This document describes version 0.001 of SQLite::KeyValueStore::Simple (from Perl distribution SQLite-KeyValueStore-Simple), released on 2021-06-18.
+This document describes version 0.002 of SQLite::KeyValueStore::Simple (from Perl distribution SQLite-KeyValueStore-Simple), released on 2021-06-18.
 
 =head1 SYNOPSIS
 
 From Perl:
 
  use SQLite::KeyValueStore::Simple qw(
+     dump_sqlite_kvstore
      list_sqlite_kvstore_keys
      get_sqlite_kvstore_value
      set_sqlite_kvstore_value
@@ -389,7 +434,7 @@ Usage:
 
 Check whether a key exists.
 
-This function is not exported.
+This function is not exported by default, but exportable.
 
 Arguments ('*' denotes required arguments):
 
@@ -427,6 +472,46 @@ Return value:  (any)
 
 
 
+=head2 dump_sqlite_kvstore
+
+Usage:
+
+ dump_sqlite_kvstore(%args) -> [$status_code, $reason, $payload, \%result_meta]
+
+Dump content of key-value store as hash.
+
+This function is not exported by default, but exportable.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<path> => I<filename>
+
+Database path.
+
+If not specified, will default to $HOME/kvstore.db. If file does not exist, will
+be created by DBD::SQLite.
+
+If you want an in-memory database (that will be destroyed after your process
+exits), use C<:memory:>.
+
+
+=back
+
+Returns an enveloped result (an array).
+
+First element ($status_code) is an integer containing HTTP-like status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+($reason) is a string containing error message, or something like "OK" if status is
+200. Third element ($payload) is the actual result, but usually not present when enveloped result is an error response ($status_code is not 2xx). Fourth
+element (%result_meta) is called result metadata and is optional, a hash
+that contains extra information, much like how HTTP response headers provide additional metadata.
+
+Return value:  (any)
+
+
+
 =head2 get_sqlite_kvstore_value
 
 Usage:
@@ -437,7 +522,7 @@ Get the current value of a key, will return undef if key does not exist.
 
 CLI will exit non-zero (1) when key does not exist.
 
-This function is not exported.
+This function is not exported by default, but exportable.
 
 Arguments ('*' denotes required arguments):
 
@@ -489,7 +574,7 @@ Usage:
 
 List existing keys in the key-value store.
 
-This function is not exported.
+This function is not exported by default, but exportable.
 
 Arguments ('*' denotes required arguments):
 
@@ -533,7 +618,7 @@ Will automatically create the key if not already exists.
 
 Will return the old value (or C<undef> if key previously did not exist).
 
-This function is not exported.
+This function is not exported by default, but exportable.
 
 Arguments ('*' denotes required arguments):
 
@@ -569,6 +654,8 @@ be created by DBD::SQLite.
 If you want an in-memory database (that will be destroyed after your process
 exits), use C<:memory:>.
 
+=item * B<quiet> => I<bool>
+
 =item * B<value>* => I<str>
 
 Value.
